precision highp float;

uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform sampler2D normalTexture;

uniform mat4 world; //GlobalConstBuffer contents

uniform mat4 local; //PerConstDrawBuffer
uniform vec4 sunDirection;
uniform vec4 fresnelAmbientTerm;
uniform vec4 scaleFactor;
uniform float animationPhase;
uniform float cloudOpacity;
uniform float cloudHeight;
uniform float cloudShadowOpacity;

varying vec4 position;
varying vec4 uv;
varying vec4 normal;
varying vec4 tangent;
varying vec4 binormal;
varying vec4 viewDir;
varying vec4 lightDirection;

void main()
{
	vec3 normalMap = 2.0 * texture2D(normalTexture, uv.xy).rgb - 1.0;
	vec3 lNormal = normalize( normal.xyz * normalMap.b + 0.15 * tangent.xyz * normalMap.r + 0.15 * binormal.xyz * normalMap.g );
	vec3 lLightDirection = sunDirection.xyz;

	vec3 lViewDir = normalize( viewDir.xyz );
	vec4 specularMap = texture2D(specularTexture, uv.xy);
	vec3 reflectedLight = -reflect( lLightDirection, lNormal );

	float lambertSaturated = clamp( dot( lNormal, lLightDirection ), 0.0, 1.0 );
	vec3 lambertianFactor = vec3(lambertSaturated, lambertSaturated, lambertSaturated) + vec3( 0.05, 0.1, 0.15 ); // SE: hack ambient

	float power = clamp( dot( reflectedLight, lViewDir ), 0.0, 1.0 );
	power *= power;
	power *= power;
	power *= power;
	vec3 specular = mix( vec3( 0.05, 0.1, 0.15 ), vec3( 1.0, 1.0, 1.0 ), clamp( 1.5 * power * power, 0.0, 1.0 ) );
	vec3 specularComponent = clamp( specular * specularMap.xyz, 0.0, 1.0 );

	vec4 textureSample = texture2D( diffuseTexture, uv.xy );
	vec3 diffuse = lambertianFactor * textureSample.xyz;
	//if( textureSample.a - 0.5 < 0.0 )
	//{
	//	discard;
	//}
	vec3 final = diffuse + specularComponent;
	gl_FragColor = vec4(final, 1.0);
}
