precision mediump float;

uniform vec3 uLightDirection;
uniform vec3 uExposure;

const float g = -0.990;
const float g2 = g * g;

varying vec3 c0;
varying vec3 c1;
varying vec3 t0;
varying vec3 temp;

// Calculates the Mie phase function
float getMiePhase(float fCos, float fCos2, float g, float g2)
{
	return 1.5 * ((1.0 - g2) / (2.0 + g2)) * (1.0 + fCos2) / pow(1.0 + g2 - 2.0*g*fCos, 1.5);
}

// Calculates the Rayleigh phase function, the original paper suggests that this function be skipped and replaced with a constant of 1.0
float getRayleighPhase(float fCos2)
{
	//return 1.0;
	return 0.75 + 0.75*fCos2;
}

void main()
{

	
	vec3 viewDir = t0;//normalize(t0);
	float fCos = dot(uLightDirection, viewDir) / length(viewDir);
	float fCos2 = fCos*fCos;
	
	vec4 final;
	//final.rgb = getRayleighPhase(fCos2) * c0 + getMiePhase(fCos, fCos2, g, g2) * c1;
	final.rgb = 1.0 * c0 + getMiePhase(fCos, fCos2, g, g2) * c1;

	//Recommended exposure, looks nice but is slow.
	//final.rgb = vec3(1.0) - exp(vec3(-2.0) * final.rgb);
	
	//S curve exposure
	//final.rgb = final.rgb / (final.rgb + uExposure);
	
	//gamma correction back to 2.0 space doesn't seem to be required but left here just in case
	//final.rgb = sqrt(final.rgb);
	
	//filmic tone mapping
	final.rgb *= uExposure * vec3(0.25);
	final.rgb = max(vec3(0.0),final.rgb - vec3(0.004));
	final.rgb = (final.rgb*(vec3(6.2)*final.rgb+vec3(0.5))) / (final.rgb*(vec3(6.2)*final.rgb+vec3(1.7))+vec3(0.06));
	
	final.a = 1.0;
	//final.rgb = temp;
	gl_FragColor = final;
	//gl_FragColor = vec4(1.0,0.0,0.0,1.0);
}