precision highp float;

uniform vec3 uLightDirection;
uniform vec3 uExposure;

const float g = -0.999; // Controls sun glow size, must not be less than -1.0!
const float g2 = g * g;

varying vec3 normal;
varying vec3 viewVector;

uniform sampler2D atmos_frontcolor;

// Calculates the Mie phase function
float getMiePhase(float fCos, float fCos2, float g, float g2)
{
    return 1.5 * ((1.0 - g2) / (2.0 + g2)) * (1.0 + fCos2) / (1.0 + g2 - 2.0 * g * fCos);
}

float schlick(float power, vec3 a, vec3 b)
{
    float p = 1.0 - dot(a,b);
    p = p * p;
    p = p * p;

    return power + (1.0 - power) * p;

    //float result = power + (1.0 - power) * pow((1.0 - dot(a,b)), 5.0);
    //return result;
}

void main()
{
    vec3 viewDir = normalize(viewVector);
    
    float fCos = dot(uLightDirection, viewDir);
    float fCos2 = fCos * fCos;
    
    float tc_light = dot(uLightDirection, normal);
    tc_light = (tc_light * 0.5 + 0.5) * 0.85;
    
    float fres = schlick(0.01, viewDir, normal);
    float tc_view = 1.2 - clamp(fres, 0.0, 1.0);
    vec2 lookup_tc = vec2(tc_view, tc_light);
    
    vec3 c0 = texture2D(atmos_frontcolor, lookup_tc).rgb;
    c0 *= c0;
    
    vec4 final = vec4(c0 + clamp(getMiePhase(fCos, fCos2, g, g2) * c0, 0.0, 1.0), 1.0);

    gl_FragColor = final;//vec4(final.r * fade, final.g * fade, final.b * fade, 1.0);

    //gl_FragColor = vec4(fade, fade, fade, 1.0);
}