precision mediump float;

uniform vec3 uLightDirection;
uniform vec3 uAmbient;
uniform vec3 uExposure;

//Unique variables
//const float bias = 0.001;
uniform float uPlanetRadius2;

const float planetRadius = 0.5;
//const float uPlanetRadius2 = (planetRadius * planetRadius); // Constant, pass in

//Colour tint for light passing through the rings...
const vec3 ringBackScatterColour = vec3(1.0, 0.81, 0.642);

varying vec2 texCoord;
varying float distance2;
varying vec3 wPos;
varying vec3 viewDir;
varying vec3 normal;
varying vec3 radialNormal;

varying float frontBackScatterWeight;
varying float unlitWeight;
varying float scatterLight;

varying float distance2MinusOuterRadius2;

uniform sampler2D ringDiffuseTex;
uniform sampler2D ringScatterTex;

void main()
{
	vec4 ringDiffuse = texture2D(ringDiffuseTex, texCoord);
	
	//Shadow
	float B = 2.0 * dot(wPos, uLightDirection);
	float fDet = max(0.0, B*B - 4.0 * distance2MinusOuterRadius2);
	float fShadow = 0.5 * (-B - sqrt(fDet));
	fShadow = max(0.0, fShadow);
	vec3 pointOnSphere = wPos + (uLightDirection * fShadow);
	float distToPlanet = dot(pointOnSphere, pointOnSphere);
	
	//Hard shadows
	fShadow = step(uPlanetRadius2, distToPlanet);
	
	vec3 final = mix(ringBackScatterColour * ringDiffuse.rgb, ringDiffuse.rgb, frontBackScatterWeight);

	float ring_light = mix(ringDiffuse.g, ringDiffuse.r, frontBackScatterWeight);
	ring_light = mix(ringDiffuse.b, ring_light, unlitWeight);
	ring_light *= scatterLight;
	
	vec3 shadowAndAmbient =  (fShadow + ((1.0 - fShadow) * uAmbient));
	final *= ring_light * shadowAndAmbient;
	
	final += ringDiffuse.rgb * shadowAndAmbient;
	//final = ringDiffuse.rgb * (fShadow + ((1.0 - fShadow) * uAmbient));// * 0.5);

	gl_FragColor = vec4(final, ringDiffuse.a);
	//gl_FragColor = vec4(fShadow, fShadow, fShadow, 1);
}