precision highp float;

uniform vec3 uLightDirection;
//uniform vec3 uAmbientColor;
uniform vec3 uExposure;

uniform float uCloudOpacity;

varying vec2 texCoord0;
varying vec3 normal;
varying vec3 viewDir;

uniform sampler2D diffuseTexture;
uniform sampler2D additionalTexture;
uniform sampler2D gradientShadeTexture;
uniform sampler2D atmosphereGradientTexture;
uniform sampler2D nightTexture;
uniform sampler2D specularGradientTexture;
uniform sampler2D mathTexture;

const float pi = 3.142;

const vec3 lightColor = vec3(1.0, 0.9, 0.2);
const vec4 specularColor = vec4(0.8, 0.9, 1.0, 1.0);

const float shininess = 96.0;

const vec4 atmosphereColor = vec4(70.0 / 255.0, 102.0 / 255.0, 163.0 / 255.0, 1.0);

void main()
{
	vec4 final;

	vec3 n = normalize(normal); //the normal must be normalized again, otherwise it will transition linearly from one vertex to the next
    float tc_light = dot(n, uLightDirection);
	tc_light = tc_light * 0.5 + 0.5;
	
	vec3 halfVector = normalize(uLightDirection + viewDir);
	float NdotHV = max(dot(n, halfVector), 0.0);
	
	float specularity = pow(NdotHV, shininess);
	vec4 specular = specularColor * specularity;
	//vec4 specular = texture2D(specularGradientTexture, vec2(NdotHV, 0));
	
	vec2 shadingTexLookup = vec2(tc_light, 0);
	vec4 shading = texture2D(gradientShadeTexture, shadingTexLookup);

    float tc_view = dot(normal, viewDir);
    //tc_view = tc_view * 0.5 + 0.5;
    tc_view = clamp(tc_view, 0.0, 1.0);

	vec2 atmosTexLookUp = vec2(tc_view, 0);
	vec4 atmosphere = texture2D(atmosphereGradientTexture, atmosTexLookUp) * shading;
	
	vec4 land = texture2D(diffuseTexture, texCoord0);
	
	float atmosValue = texture2D(mathTexture, atmosTexLookUp).g * shading.r * 0.5;
	vec4 landAndAtmosphericHaze = mix(land, atmosphereColor, atmosValue);
	
	vec4 additional = texture2D(additionalTexture, texCoord0);
	
	vec4 landAndSpecular = landAndAtmosphericHaze + specular * additional.b;
	
	vec4 nightLights;
	float nightLightsMultiplier = 1.0 - ((shading.r - 0.2) * 1.2);
	nightLights.rgb = lightColor * additional.g * nightLightsMultiplier;

	float cloudValue = additional.r * uCloudOpacity;
	
	vec4 diff = landAndSpecular * (1.0 - cloudValue) + cloudValue; //mix the clouds in over the top

	final = diff * shading + atmosphere + nightLights;
	
	final.a = 1.0;
	gl_FragColor = final;

}