precision mediump float;

uniform vec3 uLightDirection;
uniform vec3 uAmbientColor;
uniform vec3 uExposure;


varying vec2 TexCoord;
varying vec3 normal;
varying vec3 viewDir;
//varying vec3 wPos; // Only used by saturn surface shader

uniform sampler2D diffuseTexture;
uniform sampler2D atmos_frontcolor;
uniform sampler2D atmos_attenuate;
uniform sampler2D nightTexture;


//Note, this method differs from that used in other atmospheric planet shaders, it also
//returns NdotL which is then used to blend between night and day textures.
void getLookupLighting(sampler2D atmos_frontcolor_sampler, 
						sampler2D atmos_attenuate_sampler, 
						vec3 normal, 
						vec3 viewDir,
						out vec3 c0,
						out vec3 c1,
						out float NdotL)
{
	NdotL = dot(normal,uLightDirection);
	float tc_light = NdotL;
	tc_light = (1.0 - (tc_light * 0.5 + 0.5));
	
	vec3 normalised_viewDir = normalize(viewDir);
	float tc_view = dot(normal,normalised_viewDir);
	tc_view = 1.0-clamp(tc_view,0.0,1.0);

	vec2 atmosTexLookup = vec2(tc_view,tc_light);
	c0 = texture2D(atmos_frontcolor_sampler,atmosTexLookup).rgb;
	c0 *= c0;
	c1 = texture2D(atmos_attenuate_sampler,atmosTexLookup).rgb;
	c1 *= c1;
}

void main()
{
	vec3 c0;
	vec3 c1;
	float NdotL; // Reuse ndotl as a mask for night lighting;
	
	getLookupLighting(atmos_frontcolor,atmos_attenuate,normal,viewDir,c0,c1,NdotL);
	
	vec3 v3DiffuseColor = texture2D(diffuseTexture,TexCoord).rgb;
	v3DiffuseColor = v3DiffuseColor * v3DiffuseColor;
	
	vec3 v3NightColor = texture2D(nightTexture,TexCoord).rgb;
	v3NightColor = v3NightColor * v3NightColor;
	
//	float dayNightBlend = clamp(dot(c1,vec3(0.2125,0.7154,0.0721)),
	//							0.0,
	//							1.0);
	float dayNightBlend = clamp(NdotL * 3.151,
								0.0,
								1.0);						
	//v3DiffuseColor = mix(v3NightColor,v3DiffuseColor,dayNightBlend);

	vec4 final;
	final.rgb = (c0 + v3DiffuseColor * c1); 
	//final.rgb += v3DiffuseColor * uAmbientColor;// * vec3((1.0 - dayNightBlend)); // ambient only in night areas.
	final.rgb += v3NightColor * uAmbientColor * vec3((1.0 - dayNightBlend)); // ambient only in night areas.
	
	//final.rgb = c0 + vec3(1.0) * c1;
	//Recommended exposure, looks nice but it's slow.
	//final.rgb = vec3(1.0) - exp(vec3(-2.0) * final.rgb);
	
	//S curve exposure
	//final.rgb = abs(final.rgb / (final.rgb + uExposure));
	
	//gamma correction back to 2.0 space
	//final.rgb = sqrt(final.rgb);
	
	//Filmic tonemapping
	final.rgb *= uExposure;
	final.rgb = max(vec3(0.0),final.rgb - vec3(0.004));
	final.rgb = (final.rgb*(vec3(6.2)*final.rgb+vec3(0.5))) / (final.rgb*(vec3(6.2)*final.rgb+vec3(1.7))+vec3(0.06));
	
	final.a = 1.0;
	
	gl_FragColor = final;
}