precision mediump float;

uniform vec3 uLightDirection;
uniform vec3 uRingPlaneNormal; // Represents the "up" or plane normal of the plane the planets rings lie on

//uniform float uLightDirectionDotRingPlaneNormal;

varying vec2 texCoord0;
varying vec3 normal;
varying vec3 viewDir;
varying vec3 collisionPoint;

//varying float pointDistance;

uniform sampler2D diffuseTexture;
uniform sampler2D gradientShadeTexture;
uniform sampler2D ringTexture; 

uniform float uAmbient;

const float innerRadius = 1.1611;
const float innerRadius2 = innerRadius * innerRadius;
const float outerRadius = 2.3258;
const float outerRadius2 = outerRadius * outerRadius;
const float inverseRingWidth2 = 1.0 / (outerRadius2 - innerRadius2);

void main()
{
    float distSquared = dot(collisionPoint, collisionPoint);
    
    float tc = (distSquared - innerRadius2) * inverseRingWidth2;
    float discShadow = clamp(tc, 0.0, 1.0);
    
    vec2 shadowUV = vec2(discShadow, 0.0);
    float shadowFac = texture2D(ringTexture, shadowUV).r;
    
    float tc_light = dot(normal, uLightDirection);
    tc_light = tc_light * 0.5 + 0.5;
    
    vec2 shadingTexLookup = vec2(tc_light, 0);
    vec4 shading = texture2D(gradientShadeTexture, shadingTexLookup);
    
    vec4 diff = texture2D(diffuseTexture, texCoord0);
    
    shading = shading * shadowFac + uAmbient;
    
    vec4 final = diff * shading;

    final.a = 1.0;
    
    gl_FragColor = final;
}