precision mediump float;

uniform vec3 uLightDirection;
uniform vec3 uAmbientColor;
uniform vec3 uExposure;

//Unique variables
const float bias = 0.001;
const float shadowSoftness = 0.6;
const float planetRadius = 0.5;

const float fOuterRadius = planetRadius; // Constant, pass in
const float fOuterRadius2 = (planetRadius * planetRadius); // Constant, pass in

//Colour tint for light passing through the rings...
const vec4 ringBackScatterColour = vec4(1.0,0.81,0.642,1.0);


varying vec2 TexCoord;
varying float dist;
varying vec3 wPos;
varying vec3 viewDir;
varying vec3 normal;
varying vec3 radialNormal;

uniform sampler2D ringDiffuseTex;
uniform sampler2D ringScatterTex;

float getNearIntersection(vec3 v3Pos, vec3 v3Ray, float fDistance2, float fRadius2)
{
	float B = 2.0 * dot(v3Pos, v3Ray);
	float C = fDistance2 - fRadius2;
	float fDet = max(0.0, B*B - 4.0 * C);
	return 0.5 * (-B - sqrt(fDet));
}

void main() {
	vec4 ringDiffuse = texture2D(ringDiffuseTex,TexCoord);
	ringDiffuse *= ringDiffuse;
	
	vec4 ringFBUA = texture2D(ringScatterTex,TexCoord);
	vec4 final;
	
	//Shadow
	float distance2 = dist * dist;
	float fShadow = getNearIntersection(wPos, uLightDirection, distance2, fOuterRadius2);
	fShadow = max(0.0,fShadow);
	vec3 pointOnSphere = wPos + (uLightDirection * fShadow);
	float distToPlanet = dot(pointOnSphere,pointOnSphere);//Get squared length instead of length(pointOnSphere);
	
	//Hard shadows
	fShadow = (distToPlanet) > (fOuterRadius2 + bias) ? 1.0:0.0;	
	
	//More realistic front back scatter
	float frontBackScatterWeight = dot(viewDir,uLightDirection);

	//Alternate front back scatter, more expensive
	//vec3 refl = reflect(uLightPos,radialNormal);
	//frontBackScatterWeight = dot(viewDir,refl);
	
	frontBackScatterWeight = (frontBackScatterWeight * 0.5) + 0.5;
	
	
	float unlitWeight = dot(uLightDirection,normal); 
	
	unlitWeight = ((unlitWeight * 0.5) + 0.5);
	float scatterLight = dot(uLightDirection,radialNormal);
	scatterLight = ((scatterLight * 0.5) + 0.5);
	
	final.rgb = mix(ringBackScatterColour.rgb * ringDiffuse.rgb,ringDiffuse.rgb,frontBackScatterWeight);

	float ring_light = mix(ringFBUA.g,ringFBUA.r,frontBackScatterWeight);
	ring_light = mix(ringFBUA.b,ring_light,unlitWeight);
	ring_light *= scatterLight;
	final.rgb *= ring_light * fShadow;
	
	final.rgb += ringDiffuse.rgb * uAmbientColor; // ambient
	
	//Expensive exposure
	//final.rgb = vec3(1.0) - exp(v3Exposure * final.rgb);
	
	//S curve exposure
	//final.rgb = final.rgb / (final.rgb + uExposure);
	
	//gamma correction back to 2.0 space
	//final.rgb = sqrt(final.rgb);
	
	//filmic tone mapping
	final.rgb *= uExposure;
	final.rgb = max(vec3(0.0),final.rgb - vec3(0.004));
	final.rgb = (final.rgb*(vec3(6.2)*final.rgb+vec3(0.5))) / (final.rgb*(vec3(6.2)*final.rgb+vec3(1.7))+vec3(0.06));
	
	
	final.a = ringDiffuse.a;

	gl_FragColor = final;
	//gl_FragColor = vec4(fShadow,0,0,1);
}